package com.parivedasolutions.zaastra.gwt.client;

import java.util.ArrayList;
import java.util.List;

import net.customware.gwt.presenter.client.EventBus;
import net.customware.gwt.presenter.client.place.PlaceManager;
import net.customware.gwt.presenter.client.widget.WidgetDisplay;
import net.customware.gwt.presenter.client.widget.WidgetPresenter;

import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;
import com.parivedasolutions.zaastra.gwt.client.event.ShowI18nExampleEvent;
import com.parivedasolutions.zaastra.gwt.client.event.ShowRpcExampleEvent;
import com.parivedasolutions.zaastra.gwt.client.event.ShowWidgetExampleEvent;
import com.parivedasolutions.zaastra.gwt.client.i18n.I18nExamplePresenter;
import com.parivedasolutions.zaastra.gwt.client.rpc.RpcExamplePresenter;
import com.parivedasolutions.zaastra.gwt.client.widget.WidgetExamplePresenter;

public class AppPresenter extends WidgetPresenter<AppPresenter.Display> {

	public interface Display extends WidgetDisplay {

		public void addI18nExampleWidget(Widget widget);

		public void addRpcExampleWidget(Widget widget);

		public void addWidgetExampleWidget(Widget widget);

		public HandlerRegistration addSelectionHandler(SelectionHandler<Integer> handler);

		public void showDisplay(int index);
	}

	private List<WidgetPresenter<? extends WidgetDisplay>> presenterList = new ArrayList<WidgetPresenter<? extends WidgetDisplay>>();
	private PlaceManager placeManager;
	private RpcExamplePresenter rpcExamplePresenter;
	private WidgetExamplePresenter widgetExamplePresenter;
	private I18nExamplePresenter i18nExamplePresenter;
	private EventBus eventBus;

	@Inject
	public AppPresenter(Display display, EventBus eventBus, RpcExamplePresenter rpcExamplePresenter, I18nExamplePresenter i18nExamplePresenter,
			WidgetExamplePresenter widgetExamplePresenter, PlaceManager placeManager) {
		super(display, eventBus);
		this.rpcExamplePresenter = rpcExamplePresenter;
		this.widgetExamplePresenter = widgetExamplePresenter;
		this.i18nExamplePresenter = i18nExamplePresenter;
		this.placeManager = placeManager;
		this.eventBus = eventBus;
	}

	@Override
	protected void onBind() {

		rpcExamplePresenter.bind();
		display.addRpcExampleWidget(rpcExamplePresenter.getDisplay().asWidget());
		presenterList.add(rpcExamplePresenter);
		eventBus.addHandler(ShowRpcExampleEvent.TYPE, new ShowRpcExampleEvent.handler() {
			public void showDisplay() {
				if (presenterList.size() >= 0) {
					display.showDisplay(0);
				}
			}
		});

		i18nExamplePresenter.bind();
		display.addI18nExampleWidget(i18nExamplePresenter.getDisplay().asWidget());
		presenterList.add(i18nExamplePresenter);
		eventBus.addHandler(ShowI18nExampleEvent.TYPE, new ShowI18nExampleEvent.handler() {
			public void showDisplay() {
				if (presenterList.size() >= 1) {
					display.showDisplay(1);
				}
			}
		});

		widgetExamplePresenter.bind();
		display.addWidgetExampleWidget(widgetExamplePresenter.getDisplay().asWidget());
		presenterList.add(widgetExamplePresenter);
		eventBus.addHandler(ShowWidgetExampleEvent.TYPE, new ShowWidgetExampleEvent.handler() {
			public void showDisplay() {
				if (presenterList.size() >= 2) {
					display.showDisplay(2);
				}
			}
		});

		display.addSelectionHandler(new SelectionHandler<Integer>() {

			public void onSelection(SelectionEvent<Integer> event) {
				Integer item = event.getSelectedItem();
				presenterList.get(item).revealDisplay();
				placeManager.fireCurrentPlace();
			}
		});

	}

	@Override
	protected void onRevealDisplay() {
		// TODO Auto-generated method stub

	}

	@Override
	protected void onUnbind() {
		// TODO Auto-generated method stub

	}

}
