package com.parivedasolutions.zaastra.gwt.client.i18n;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.FlexTable.FlexCellFormatter;
import com.google.inject.Inject;

/**
 * Adapted from
 * http://gwt.google.com/samples/Showcase/Showcase.html#!CwPluralFormsExample
 * 
 */
public class I18nExampleView extends Composite implements I18nExamplePresenter.Display {

	FlowPanel panel = new FlowPanel();

	/**
	 * The {@link TextBox} where the user enters argument 0.
	 */
	private TextBox arg0Box;

	/**
	 * The {@link Label} used to display the message.
	 */
	private Label formattedMessage;

	private final AlternateMessages altMessages;
	private static final Messages MESSAGES = (Messages) GWT.create(Messages.class);

	@Inject
	public I18nExampleView(Messages messages, AlternateMessages altMessages) {
		this.altMessages = altMessages;
		Label title = new Label(MESSAGES.title());
		title.addStyleName("title");
		panel.add(title);
		FlowPanel messageContainer = new FlowPanel();
		messageContainer.addStyleName("messageContainer");
		Label messageLabel = new Label(MESSAGES.getLyrics());
		messageContainer.add(messageLabel);
		panel.add(messageContainer);

		Label title2 = new Label(messages.title2());
		title2.addStyleName("title");
		panel.add(title2);
		// Start adaptation of Google Demo
		// Use a FlexTable to layout the content
		FlexTable layout = new FlexTable();
		FlexCellFormatter formatter = layout.getFlexCellFormatter();
		layout.setCellSpacing(5);

		// Add argument 0
		arg0Box = new TextBox();
		arg0Box.setText("13");
		layout.setHTML(2, 0, MESSAGES.pluralFormsExampleArg0Label());
		layout.setWidget(2, 1, arg0Box);

		// Add the formatted message
		formattedMessage = new Label();
		layout.setHTML(5, 0, messages.pluralFormsExampleFormattedLabel());
		layout.setWidget(5, 1, formattedMessage);
		formatter.setVerticalAlignment(5, 0, HasVerticalAlignment.ALIGN_TOP);

		// Add handlers to all of the argument boxes
		KeyUpHandler keyUpHandler = new KeyUpHandler() {
			public void onKeyUp(KeyUpEvent event) {
				updateMessage();
			}
		};
		arg0Box.addKeyUpHandler(keyUpHandler);

		// Return the layout Widget
		updateMessage();
		panel.add(layout);

		initWidget(panel);
	}

	/**
	 * Update the formatted message.
	 */
	private void updateMessage() {
		try {
			int count = Integer.parseInt(arg0Box.getText().trim());
			formattedMessage.setText(altMessages.treeCount(count));
		} catch (NumberFormatException e) {
			// Ignore.
		}
	}
}
