package com.parivedasolutions.zaastra.gwt.client.rpc;

import java.util.Date;

import net.customware.gwt.presenter.client.EventBus;
import net.customware.gwt.presenter.client.widget.WidgetDisplay;
import net.customware.gwt.presenter.client.widget.WidgetPresenter;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.HasKeyUpHandlers;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasEnabled;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.HasValue;
import com.google.inject.Inject;
import com.parivedasolutions.zaastra.gwt.shared.FieldVerifier;
import com.parivedasolutions.zaastra.gwt.shared.RpcValueObject;

/**
 * Demonstrates the GWT Remote Procedure Call framework
 * 
 */
public class RpcExamplePresenter extends WidgetPresenter<RpcExamplePresenter.Display> {

	public interface Display extends WidgetDisplay {
		public HasText getErrorLabel();

		public HasValue<String> getNameField();

		public HasText getTextToServerField();

		public HasText getResponseField();

		public HasValue<Date> getDateField();

		public HasEnabled getSendEnabler();

		public HasClickHandlers getSendClick();

		public HasKeyUpHandlers getFieldKeyUp();

		public void onCallSuccess(String message);

		public void onCallFailure(String message);
	}

	/**
	 * Create a remote service proxy to talk to the server-side Greeting
	 * service.
	 */
	private final GreetingServiceAsync greetingService;

	/**
	 * The message displayed to the user when the server cannot be reached or
	 * returns an error.
	 */
	private static final String SERVER_ERROR = "An error occurred while attempting to contact the server. Please check your network connection and try again.";

	@Inject
	public RpcExamplePresenter(Display display, EventBus eventBus, GreetingServiceAsync greetingService) {
		super(display, eventBus);
		this.greetingService = greetingService;
	}

	// Create a handler for the sendButton and nameField
	class RpcHandler implements ClickHandler, KeyUpHandler {
		/**
		 * Fired when the user clicks on the sendButton.
		 */
		public void onClick(ClickEvent event) {
			sendDataToServer();
		}

		/**
		 * Fired when the user types in the nameField.
		 */
		public void onKeyUp(KeyUpEvent event) {
			if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER) {
				sendDataToServer();
			}
		}

		/**
		 * Send the name from the nameField to the server and wait for a
		 * response.
		 */
		private void sendDataToServer() {
			// First, we validate the input.
			display.getErrorLabel().setText("");
			String textToServer = display.getNameField().getValue();
			if (!FieldVerifier.isValidName(textToServer)) {
				display.getErrorLabel().setText(FieldVerifier.ERROR_MESSAGE);
				return;
			}

			// Then, we send the input to the server.
			display.getSendEnabler().setEnabled(false);
			display.getTextToServerField().setText(textToServer);
			display.getResponseField().setText("");
			greetingService.sendToServer(new RpcValueObject(textToServer, display.getDateField().getValue()), new AsyncCallback<String>() {
				public void onFailure(Throwable caught) {
					// Show the RPC error message to the user
					display.onCallFailure(SERVER_ERROR);
				}

				public void onSuccess(String result) {
					// Show the RPC result to the user
					display.onCallSuccess(result);
				}
			});

		}
	}

	@Override
	protected void onBind() {
		RpcHandler handler = new RpcHandler();
		display.getSendClick().addClickHandler(handler);
		display.getFieldKeyUp().addKeyUpHandler(handler);
	}

	@Override
	protected void onRevealDisplay() {

	}

	@Override
	protected void onUnbind() {

	}

}
