package com.parivedasolutions.zaastra.gwt.client.rpc;

import java.util.Date;

import net.customware.gwt.presenter.client.widget.WidgetDisplay;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.HasKeyUpHandlers;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasEnabled;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.datepicker.client.DateBox;
import com.google.gwt.user.datepicker.client.DateBox.DefaultFormat;
import com.google.gwt.user.datepicker.client.DateBox.Format;
import com.parivedasolutions.zaastra.gwt.shared.Constants;

public class RpcExampleView extends Composite implements RpcExamplePresenter.Display {

	private FlowPanel panel = new FlowPanel();

	private Button dialogCloseButton;
	private DialogBox dialogBox;

	private Label textToServerLabel;
	private HTML serverResponseLabel;
	private Label errorLabel;

	private Button sendButton;
	private TextBox nameField;
	private DateBox dateBox;

	public RpcExampleView() {

		Label title = new Label("RPC Example");
		title.addStyleName("title");
		panel.add(title);
		sendButton = new Button("Send");
		nameField = new TextBox();
		nameField.setText("Name");
		createDateBox();
		errorLabel = new Label();

		// We can add style names to widgets
		sendButton.addStyleName("sendButton");

		createWidgetContainer(nameField, "nameFieldContainer");
		createWidgetContainer(dateBox, "datePickerContainer");
		createWidgetContainer(sendButton, "sendButtonContainer");
		createWidgetContainer(errorLabel, "errorLabelContainer");

		// Focus the cursor on the name field when the app loads
		nameField.setFocus(true);
		nameField.selectAll();

		// Create the popup dialog box
		createDialogBox();

		initWidget(panel);
	}

	private void createDateBox() {
		Format format = new DefaultFormat(DateTimeFormat.getFormat(Constants.DATE_FORMAT_PATTERN));
		dateBox = new DateBox();
		dateBox.setFormat(format);
	}

	private void createWidgetContainer(final Widget widget, String styleName) {
		FlowPanel container = new FlowPanel();
		container.addStyleName(styleName);
		container.add(widget);
		panel.add(container);
	}

	private void createDialogBox() {
		dialogBox = new DialogBox();
		dialogBox.setText("Remote Procedure Call");
		dialogBox.setAnimationEnabled(true);
		dialogCloseButton = new Button("Close");
		// We can set the id of a widget by accessing its Element
		dialogCloseButton.getElement().setId("closeButton");
		textToServerLabel = new Label();
		serverResponseLabel = new HTML();
		VerticalPanel dialogVPanel = new VerticalPanel();
		dialogVPanel.addStyleName("dialogVPanel");
		dialogVPanel.add(new HTML("<b>Sending name to the server:</b>"));
		dialogVPanel.add(textToServerLabel);
		dialogVPanel.add(new HTML("<br><b>Server replies:</b>"));
		dialogVPanel.add(serverResponseLabel);
		dialogVPanel.setHorizontalAlignment(VerticalPanel.ALIGN_RIGHT);
		dialogVPanel.add(dialogCloseButton);
		dialogBox.setWidget(dialogVPanel);

		// Add a handler to close the DialogBox
		dialogCloseButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				dialogBox.hide();
				sendButton.setEnabled(true);
				sendButton.setFocus(true);
			}
		});
	}

	/**
	 * Returns this widget as the {@link WidgetDisplay#asWidget()} value.
	 */
	@Override
	public RpcExampleView asWidget() {
		return this;
	}

	public HasText getErrorLabel() {
		return errorLabel;
	}

	public HasKeyUpHandlers getFieldKeyUp() {
		return nameField;
	}

	public HasValue<String> getNameField() {
		return nameField;
	}

	public HasText getResponseField() {
		return serverResponseLabel;
	}

	public HasEnabled getSendEnabler() {
		return sendButton;
	}

	public HasClickHandlers getSendClick() {
		return sendButton;
	}

	public HasText getTextToServerField() {
		return textToServerLabel;
	}

	public void onCallFailure(String message) {
		dialogBox.setText("Remote Procedure Call - Failure");
		serverResponseLabel.addStyleName("serverResponseLabelError");
		serverResponseLabel.setHTML(message);
		dialogBox.center();
		dialogBox.show();
		dialogCloseButton.setFocus(true);
	}

	public void onCallSuccess(String message) {
		dialogBox.setText("Remote Procedure Call");
		serverResponseLabel.removeStyleName("serverResponseLabelError");
		serverResponseLabel.setHTML(message);
		dialogBox.center();
		dialogBox.show();
		dialogCloseButton.setFocus(true);
	}

	public HasValue<Date> getDateField() {
		return dateBox;
	}

}
