package com.parivedasolutions.zaastra.gwt.client.rpc;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.Date;

import net.customware.gwt.presenter.client.DefaultEventBus;
import net.customware.gwt.presenter.client.EventBus;

import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.HasKeyUpHandlers;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasEnabled;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.HasValue;
import com.parivedasolutions.zaastra.gwt.shared.RpcValueObject;

public class RpcExamplePresenterTest {

	@Mock
	RpcExamplePresenter.Display display;

	@Mock
	GreetingServiceAsync greetingService;

	@Mock
	HasClickHandlers sendClick;

	@Mock
	HasKeyUpHandlers keyUp;

	@Mock
	HasText errorLabel;

	@Mock
	HasEnabled sendEnabler;

	@Mock
	HasText textToServerField;

	@Mock
	HasText responseField;

	@Mock
	HasValue<Date> dateField;

	@Mock
	HasValue<String> nameField;

	@Captor
	private ArgumentCaptor<ClickHandler> clickHandlerCaptor;

	RpcExamplePresenter presenter;

	EventBus eventBus = new DefaultEventBus();

	@Before
	public void setup() {
		MockitoAnnotations.initMocks(this);
		presenter = new RpcExamplePresenter(display, eventBus, greetingService);
		when(display.getSendClick()).thenReturn(sendClick);
		when(display.getFieldKeyUp()).thenReturn(keyUp);
		when(display.getErrorLabel()).thenReturn(errorLabel);
		when(display.getNameField()).thenReturn(nameField);
		when(display.getSendEnabler()).thenReturn(sendEnabler);
		when(display.getTextToServerField()).thenReturn(textToServerField);
		when(display.getResponseField()).thenReturn(responseField);
		when(display.getDateField()).thenReturn(dateField);

	}

	@SuppressWarnings("unchecked")
	@Test
	public void clickSendsNameToServer() {
		//Arrange
		String expectedName = "clickName";
		Date expectedDate = new Date();
		when(nameField.getValue()).thenReturn(expectedName);
		when(dateField.getValue()).thenReturn(expectedDate);

		//Act
		presenter.bind();
		verify(sendClick).addClickHandler(clickHandlerCaptor.capture());
		clickHandlerCaptor.getValue().onClick(
			new ClickEvent() {}
			);
		
		//Assert
		RpcValueObject valueObject = new RpcValueObject(expectedName, expectedDate);
		verify(greetingService).sendToServer(eq(valueObject), any(AsyncCallback.class));
	}
}
